@@ -21,12 +21,14 @@
 #ifndef __D_EVENT__
 #define __D_EVENT__
 
+#include "doomtype.h"
+
 //
 // Event handling.
 //
 
 // Input event types.
-typedef enum
+typedef enum evtype_e
 {
   ev_keydown,
   ev_keyup,
@@ -38,20 +40,24 @@ typedef enum
   ev_joyb_up,
   ev_joystick,
   ev_joystick_state,
-
-  // Quit event. Triggered when the user clicks the "close" button
-  // to terminate the application.
-  ev_quit,
+  ev_gyro,
+  ev_text
 } evtype_t;
 
+typedef union evdata_u
+{
+  int32_t i;
+  float f;
+} evdata_t;
+
 // Event structure.
 typedef struct event_s
 {
-  evtype_t  type;
-  int       data1;    // keys / mouse/joystick buttons / left axis x
-  int       data2;    // mouse/mouse clicks / left axis y
-  int       data3;    // mouse / right axis x
-  int       data4;    // right axis y
+  evtype_t type;
+  evdata_t data1; // keys / mouse/joystick buttons / left axis x
+  evdata_t data2; // mouse/mouse clicks / left axis y
+  evdata_t data3; // mouse / right axis x
+  evdata_t data4; // right axis y
 } event_t;
 
 #define EV_RESIZE_VIEWPORT 1
@@ -69,6 +75,8 @@ typedef enum
   ga_worlddone,
   ga_screenshot,
   ga_reloadlevel,
+  ga_loadautosave,
+  ga_saveautosave,
 } gameaction_t;
 
 
@@ -93,7 +101,7 @@ typedef enum
   BT_CHANGE       = 4,
 
   // The 4bit weapon mask and shift, convenience.
-//BT_WEAPONMASK   = (8+16+32),
+  BT_WEAPONMASK_OLD = (8+16+32),
   BT_WEAPONMASK   = (8+16+32+64), // extended to pick up SSG        // phares
   BT_WEAPONSHIFT  = 3,
 
