@@ -24,7 +24,9 @@
 // of other structs: items (internal inventory),
 // animation states (closely tied to the sprites
 // used to represent them, unfortunately).
+#include "doomtype.h"
 #include "p_pspr.h"
+#include "tables.h"
 
 // In addition, the player is just a special
 // case of the generic moving object/actor.
@@ -51,6 +53,11 @@ typedef enum
 
 } playerstate_t;
 
+typedef struct
+{
+    int episode;
+    int map;
+} level_t;
 
 //
 // Player internal flags, for cheats and debug.
@@ -81,17 +88,6 @@ typedef enum
   weapswitch_raising,
 } weapswitch_t;
 
-
-// [crispy] blinking key or skull in the status bar
-typedef enum
-{
-  KEYBLINK_NONE,
-  KEYBLINK_CARD,
-  KEYBLINK_SKULL,
-  KEYBLINK_BOTH,
-  KEYBLINK_EITHER,
-} keyblink_t;
-
 //
 // Extended player object info: player_t
 //
@@ -191,10 +187,6 @@ typedef struct player_s
   // [Woof!] show centered "A secret is revealed!" message
   char*               secretmessage;
 
-  // [crispy] blinking key or skull in the status bar
-  keyblink_t          keyblinkkeys[3];
-  int                 keyblinktics;
-
   int                 btuse, btuse_tics;
 
   // [crispy] free look / mouse look
@@ -210,6 +202,17 @@ typedef struct player_s
   // DSDA UV Max category requirements
   int maxkilldiscount;
 
+  // Local angle for blending per-frame and per-tic turning.
+  angle_t ticangle, oldticangle;
+
+  int num_visitedlevels;
+  level_t *visitedlevels;
+
+  // Last used weapon (last readyweapon).
+  weapontype_t lastweapon;
+
+  weapontype_t nextweapon;
+
 } player_t;
 
 
@@ -228,7 +231,7 @@ typedef struct
   int         stime; 
   int         frags[4];
   int         score;  // current score on entry, modified on return
-  
+
 } wbplayerstruct_t;
 
 typedef struct wbstartstruct_s
@@ -241,7 +244,8 @@ typedef struct wbstartstruct_s
   // previous and next levels, origin 0
   int         last;
   int         next;   
-  int         nextep;	// for when MAPINFO progression crosses into another episode.
+  // for when MAPINFO progression crosses into another episode.
+  int         nextep;
   struct mapentry_s *lastmapinfo;
   struct mapentry_s *nextmapinfo;
     
@@ -261,6 +265,8 @@ typedef struct wbstartstruct_s
   // [FG] total time for all completed levels
   int totaltimes;
 
+  level_t *visitedlevels;
+
 } wbstartstruct_t;
 
 
