@@ -121,21 +121,10 @@ static void ScanDir(const char *dir, boolean recursion)
         const char usr_share[] = "/usr/share";
         if (strncmp(dir, usr_share, strlen(usr_share)) == 0)
         {
-            char *home_dir = M_getenv("XDG_DATA_HOME");
-
-            if (home_dir == NULL)
-            {
-                home_dir = M_getenv("HOME");
-            }
-
-            if (home_dir)
-            {
-                char *local_share = M_StringJoin(home_dir, "/.local/share");
-                char *local_dir = M_StringReplace(dir, usr_share, local_share);
-                free(local_share);
-                ScanDir(local_dir, true);
-                free(local_dir);
-            }
+            char *local_share = M_DataDir();
+            char *local_dir = M_StringReplace(dir, usr_share, local_share);
+            ScanDir(local_dir, true);
+            free(local_dir);
         }
         else if (dir[0] == '.')
         {
