@@ -21,7 +21,7 @@
 
 #include "doomtype.h"
 
-typedef struct stream_module_s
+typedef struct
 {
     boolean (*I_InitStream)(int device);
     boolean (*I_OpenStream)(void *data, ALsizei size, ALenum *format,
@@ -30,7 +30,9 @@ typedef struct stream_module_s
     void (*I_PlayStream)(boolean looping);
     void (*I_CloseStream)(void);
     void (*I_ShutdownStream)(void);
-    const char **(*I_DeviceList)(int *current_device);
+    const char **(*I_DeviceList)(void);
+    void (*BindVariables)(void);
+    const char *(*I_MusicFormat)(void);
 } stream_module_t;
 
 extern stream_module_t stream_opl_module;
@@ -38,4 +40,8 @@ extern stream_module_t stream_fl_module;
 extern stream_module_t stream_snd_module;
 extern stream_module_t stream_xmp_module;
 
+boolean I_OAL_InitStream(void);
+
+void I_OAL_ShutdownStream(void);
+
 #endif
