@@ -55,12 +55,12 @@ int I_ConsoleStdout(void)
 }
 
 static verbosity_t verbosity = VB_INFO;
-verbosity_t cfg_verbosity;
 
 #ifdef _WIN32
 static HANDLE hConsole;
 static DWORD OldMode;
 static boolean vt_mode_enabled = false;
+static UINT OldCodePage;
 
 static void EnableVTMode(void)
 {
@@ -98,6 +98,8 @@ static void RestoreOldMode(void)
 static void I_ShutdownPrintf(void)
 {
 #ifdef _WIN32
+    SetConsoleOutputCP(OldCodePage);
+
     RestoreOldMode();
 #endif
 }
@@ -105,17 +107,18 @@ static void I_ShutdownPrintf(void)
 void I_InitPrintf(void)
 {
 #ifdef _WIN32
+    OldCodePage = GetConsoleOutputCP();
+    SetConsoleOutputCP(CP_UTF8);
+
     EnableVTMode();
 #endif
 
-    verbosity = cfg_verbosity;
-
     //!
     //
     // Print debugging info with maximum verbosity.
     //
 
-    if (M_ParmExists("-verbose") || M_ParmExists("--verbose"))
+    if (M_ParmExists("-verbose"))
     {
         verbosity = VB_MAX;
     }
@@ -125,7 +128,7 @@ void I_InitPrintf(void)
     // Print with minimum verbosity.
     //
 
-    if (M_ParmExists("-quiet") || M_ParmExists("--quiet"))
+    if (M_ParmExists("-quiet"))
     {
         verbosity = VB_ERROR;
     }
