@@ -27,8 +27,6 @@
 #include "doomtype.h"
 #include "m_fixed.h"
 
-struct stream_module_s;
-
 // when to clip out sounds
 // Does not fit the large outdoor areas.
 #define S_CLIPPING_DIST (1200 << FRACBITS)
@@ -56,12 +54,6 @@ struct stream_module_s;
 
 #define SND_SAMPLERATE  44100
 
-// [FG] variable pitch bend range
-extern int pitch_bend_range;
-
-// Pitch to stepping lookup.
-extern float steptable[256];
-
 // Init at program start...
 void I_InitSound(void);
 
@@ -72,13 +64,6 @@ void I_ShutdownSound(void);
 //  SFX I/O
 //
 
-extern int forceFlipPan;
-extern int snd_resampler;
-extern int snd_module;
-extern boolean snd_hrtf;
-extern int snd_absorption;
-extern int snd_doppler;
-
 struct mobj_s;
 
 struct sfxinfo_s;
@@ -94,20 +79,21 @@ typedef struct sound_module_s
                                  int *vol, int *sep, int *pri);
     void (*UpdateSoundParams)(int channel, int vol, int sep);
     void (*UpdateListenerParams)(const struct mobj_s *listener);
-    boolean (*StartSound)(int channel, struct sfxinfo_s *sfx, int pitch);
+    boolean (*StartSound)(int channel, struct sfxinfo_s *sfx, float pitch);
     void (*StopSound)(int channel);
     boolean (*SoundIsPlaying)(int channel);
     void (*ShutdownSound)(void);
     void (*ShutdownModule)(void);
     void (*DeferUpdates)(void);
     void (*ProcessUpdates)(void);
+    void (*BindVariables)(void);
 } sound_module_t;
 
 extern const sound_module_t sound_mbf_module;
 extern const sound_module_t sound_3d_module;
 extern const sound_module_t sound_pcs_module;
 
-typedef enum snd_module_e
+enum
 {
     SND_MODULE_MBF,
     SND_MODULE_3D,
@@ -115,10 +101,10 @@ typedef enum snd_module_e
     SND_MODULE_PCS,
 #endif
     NUM_SND_MODULES
-} snd_module_t;
+};
 
 boolean I_AllowReinitSound(void);
-void I_SetSoundModule(int device);
+void I_SetSoundModule(void);
 
 // Initialize channels?
 void I_SetChannels(void);
@@ -157,6 +143,18 @@ int I_SoundID(int handle);
 //  MUSIC I/O
 //
 
+extern boolean auto_gain;
+
+typedef enum
+{
+    midiplayer_none,
+    midiplayer_native,
+    midiplayer_fluidsynth,
+    midiplayer_opl,
+} midiplayertype_t;
+
+midiplayertype_t I_MidiPlayerType(void);
+
 typedef struct
 {
     boolean (*I_InitMusic)(int device);
@@ -166,27 +164,21 @@ typedef struct
     void (*I_ResumeSong)(void *handle);
     void *(*I_RegisterSong)(void *data, int size);
     void (*I_PlaySong)(void *handle, boolean looping);
-    void (*I_UpdateMusic)(void);
     void (*I_StopSong)(void *handle);
     void (*I_UnRegisterSong)(void *handle);
-    const char **(*I_DeviceList)(int *current_device);
+    const char **(*I_DeviceList)(void);
+    void (*I_BindVariables)(void);
+    midiplayertype_t (*I_MidiPlayerType)(void);
+    const char *(*I_MusicFormat)(void);
 } music_module_t;
 
-// Music modules
 extern music_module_t music_oal_module;
-extern music_module_t music_win_module;
-extern music_module_t music_mac_module;
-
-extern int midi_player;
-
-extern struct stream_module_s *midi_stream_module;
+extern music_module_t music_mid_module;
 
 boolean I_InitMusic(void);
 void I_ShutdownMusic(void);
 
-#define DEFAULT_MIDI_DEVICE -1  // use saved music module device
-
-void I_SetMidiPlayer(int device);
+void I_SetMidiPlayer(void);
 
 // Volume.
 void I_SetMusicVolume(int volume);
@@ -204,15 +196,13 @@ void *I_RegisterSong(void *data, int size);
 // Horrible thing to do, considering.
 void I_PlaySong(void *handle, boolean looping);
 
-void I_UpdateMusic(void);
-
 // Stops a song over 3 seconds.
 void I_StopSong(void *handle);
 
 // See above (register), then think backwards
 void I_UnRegisterSong(void *handle);
 
-const char **I_DeviceList(int *current_device);
+const char **I_DeviceList(void);
 
 // Determine whether memory block is a .mid file
 boolean IsMid(byte *mem, int len);
@@ -220,6 +210,10 @@ boolean IsMid(byte *mem, int len);
 // Determine whether memory block is a .mus file
 boolean IsMus(byte *mem, int len);
 
+const char *I_MusicFormat(void);
+
+void I_BindSoundVariables(void);
+
 #endif
 
 //----------------------------------------------------------------------------
