@@ -22,10 +22,12 @@
 
 #include "m_bbox.h"
 
+#include <limits.h>
+
 void M_ClearBox (fixed_t *box)
 {
-  box[BOXTOP] = box[BOXRIGHT] = D_MININT;
-  box[BOXBOTTOM] = box[BOXLEFT] = D_MAXINT;
+  box[BOXTOP] = box[BOXRIGHT] = INT_MIN;
+  box[BOXBOTTOM] = box[BOXLEFT] = INT_MAX;
 }
 
 void M_AddToBox(fixed_t* box,fixed_t x,fixed_t y)
