@@ -24,7 +24,7 @@
 
 struct event_s;
 
-typedef void (*cheatf_v)();
+typedef void (*cheatf_v)(void);
 typedef void (*cheatf_i)(int i);
 typedef void (*cheatf_s)(char *s);
 
