@@ -27,9 +27,16 @@
 #include "m_misc.h"
 #include "z_zone.h"
 
+#include "config.h"
+#ifdef HAVE_GETPWUID
+#include <unistd.h>
+#include <sys/types.h>
+#include <pwd.h>
+#endif
+
 // Check if a file exists
 
-boolean M_FileExists(const char *filename)
+static boolean M_FileExistsNotDir(const char *filename)
 {
     FILE *fstream;
 
@@ -38,14 +45,11 @@ boolean M_FileExists(const char *filename)
     if (fstream != NULL)
     {
         fclose(fstream);
-        return true;
+        return M_DirExists(filename) == false;
     }
     else
     {
-        // If we can't open because the file is a directory, the
-        // "file" exists at least!
-
-        return errno == EISDIR;
+        return false;
     }
 }
 
@@ -111,7 +115,7 @@ char *M_FileCaseExists(const char *path)
     path_dup = M_StringDuplicate(path);
 
     // 0: actual path
-    if (M_FileExists(path_dup))
+    if (M_FileExistsNotDir(path_dup))
     {
         return path_dup;
     }
@@ -123,7 +127,7 @@ char *M_FileCaseExists(const char *path)
     // 1: lowercase filename, e.g. doom2.wad
     M_StringToLower(filename);
 
-    if (M_FileExists(path_dup))
+    if (M_FileExistsNotDir(path_dup))
     {
         return path_dup;
     }
@@ -131,7 +135,7 @@ char *M_FileCaseExists(const char *path)
     // 2: uppercase filename, e.g. DOOM2.WAD
     M_StringToUpper(filename);
 
-    if (M_FileExists(path_dup))
+    if (M_FileExistsNotDir(path_dup))
     {
         return path_dup;
     }
@@ -142,7 +146,7 @@ char *M_FileCaseExists(const char *path)
     {
         M_StringToLower(ext + 1);
 
-        if (M_FileExists(path_dup))
+        if (M_FileExistsNotDir(path_dup))
         {
             return path_dup;
         }
@@ -153,7 +157,7 @@ char *M_FileCaseExists(const char *path)
     {
         M_StringToLower(filename + 1);
 
-        if (M_FileExists(path_dup))
+        if (M_FileExistsNotDir(path_dup))
         {
             return path_dup;
         }
@@ -231,23 +235,73 @@ const char *M_BaseName(const char *path)
     }
 }
 
+char *M_HomeDir(void)
+{
+    static char *home_dir;
+
+    if (home_dir == NULL)
+    {
+        home_dir = M_getenv("HOME");
+
+        if (home_dir == NULL)
+        {
+#ifdef HAVE_GETPWUID
+            struct passwd *user_info = getpwuid(getuid());
+            if (user_info != NULL)
+                home_dir = user_info->pw_dir;
+            else
+#endif
+                home_dir = "/";
+        }
+    }
+
+    return home_dir;
+}
+
+// Quote:
+// > $XDG_DATA_HOME defines the base directory relative to which
+// > user specific data files should be stored. If $XDG_DATA_HOME
+// > is either not set or empty, a default equal to
+// > $HOME/.local/share should be used.
+
+char *M_DataDir(void)
+{
+    static char *data_dir;
+
+    if (data_dir == NULL)
+    {
+        data_dir = M_getenv("XDG_DATA_HOME");
+
+        if (data_dir == NULL || *data_dir == '\0')
+        {
+            const char *home_dir = M_HomeDir();
+            data_dir = M_StringJoin(home_dir, "/.local/share");
+        }
+    }
+
+    return data_dir;
+}
+
 // Change string to uppercase.
 
 char M_ToUpper(const char c)
 {
-  if (c >= 'a' && c <= 'z')
-    return c + 'A' - 'a';
-  else
-    return c;
+    if (c >= 'a' && c <= 'z')
+    {
+        return c + 'A' - 'a';
+    }
+    else
+    {
+        return c;
+    }
 }
 
-void M_StringToUpper(char *text)
+void M_StringToUpper(char *str)
 {
-    char *p;
-
-    for (p = text; *p != '\0'; ++p)
+    while (*str)
     {
-        *p = M_ToUpper(*p);
+        *str = M_ToUpper(*str);
+        ++str;
     }
 }
 
@@ -255,19 +309,22 @@ void M_StringToUpper(char *text)
 
 char M_ToLower(const char c)
 {
-  if (c >= 'A' && c <= 'Z')
-    return c - 'A' + 'a';
-  else
-    return c;
+    if (c >= 'A' && c <= 'Z')
+    {
+        return c - 'A' + 'a';
+    }
+    else
+    {
+        return c;
+    }
 }
 
-void M_StringToLower(char *text)
+void M_StringToLower(char *str)
 {
-    char *p;
-
-    for (p = text; *p != '\0'; ++p)
+    while (*str)
     {
-        *p = M_ToLower(*p);
+        *str = M_ToLower(*str);
+        ++str;
     }
 }
 
@@ -505,30 +562,19 @@ void M_CopyLumpName(char *dest, const char *src)
 
 //
 // 1/18/98 killough: adds a default extension to a path
-// Note: Backslashes are treated specially, for MS-DOS.
 //
 
-char *AddDefaultExtension(char *path, const char *ext)
+char *AddDefaultExtension(const char *path, const char *ext)
 {
-    char *p = path;
-
-    while (*p++)
-        ;
-
-    while (p-- > path && *p != '/' && *p != '\\')
+    if (strrchr(M_BaseName(path), '.') != NULL)
     {
-        if (*p == '.')
-        {
-            return path;
-        }
+        // path already has an extension
+        return M_StringDuplicate(path);
     }
-
-    if (*ext != '.')
+    else
     {
-        strcat(path, ".");
+        return M_StringJoin(path, ext);
     }
-
-    return strcat(path, ext);
 }
 
 //
