@@ -23,8 +23,11 @@
 //
 //-----------------------------------------------------------------------------
 
+#include <stdint.h>
+
 #include "doomstat.h"
 #include "m_random.h"
+#include "tables.h"
 
 //
 // M_Random
