@@ -57,6 +57,9 @@ void M_Init(void);
 
 void MN_StartControlPanel(void);
 
+void MN_AddEpisode(const char *map, const char *gfx, const char *txt, char key);
+void MN_ClearEpisodes(void);
+
 void MN_ForcedLoadAutoSave(const char *msg);
 void MN_ForcedLoadGame(const char *msg); // killough 5/15/98: forced loadgames
 void MN_Trans(void);     // killough 11/98: reset translucency
