@@ -16,41 +16,41 @@
 // mus2mid.c - Ben Ryves 2006 - http://benryves.com - benryves@benryves.com
 // Use to convert a MUS file into a single track, type 0 MIDI file.
 
+#include "mus2mid.h"
 #include "doomtype.h"
 #include "m_swap.h"
 #include "memio.h"
-#include "mus2mid.h"
 
-#define NUM_CHANNELS 16
+#define NUM_CHANNELS         16
 
 #define MIDI_PERCUSSION_CHAN 9
-#define MUS_PERCUSSION_CHAN 15
+#define MUS_PERCUSSION_CHAN  15
 
 // MUS event codes
 typedef enum
 {
-    mus_releasekey = 0x00,
-    mus_presskey = 0x10,
-    mus_pitchwheel = 0x20,
-    mus_systemevent = 0x30,
+    mus_releasekey       = 0x00,
+    mus_presskey         = 0x10,
+    mus_pitchwheel       = 0x20,
+    mus_systemevent      = 0x30,
     mus_changecontroller = 0x40,
-    mus_scoreend = 0x60
+    mus_scoreend         = 0x60
 } musevent;
 
 // MIDI event codes
 typedef enum
 {
-    midi_releasekey = 0x80,
-    midi_presskey = 0x90,
-    midi_aftertouchkey = 0xA0,
-    midi_changecontroller = 0xB0,
-    midi_changepatch = 0xC0,
+    midi_releasekey        = 0x80,
+    midi_presskey          = 0x90,
+    midi_aftertouchkey     = 0xA0,
+    midi_changecontroller  = 0xB0,
+    midi_changepatch       = 0xC0,
     midi_aftertouchchannel = 0xD0,
-    midi_pitchwheel = 0xE0
+    midi_pitchwheel        = 0xE0
 } midievent;
 
 #if defined(_MSC_VER)
-#pragma pack(push, 1)
+#  pragma pack(push, 1)
 #endif
 
 // Structure to hold MUS file header
@@ -65,19 +65,19 @@ typedef PACKED_PREFIX struct
 } PACKED_SUFFIX musheader;
 
 #if defined(_MSC_VER)
-#pragma pack(pop)
+#  pragma pack(pop)
 #endif
 
 // Standard MIDI type 0 header + track header
 static const byte midiheader[] =
 {
-    'M', 'T', 'h', 'd',     // Main header
-    0x00, 0x00, 0x00, 0x06, // Header size
-    0x00, 0x00,             // MIDI type (0)
-    0x00, 0x01,             // Number of tracks
-    0x00, 0x46,             // Resolution
-    'M', 'T', 'r', 'k',        // Start of track
-    0x00, 0x00, 0x00, 0x00  // Placeholder for track length
+    'M',  'T',  'h',  'd',   // Main header
+    0x00, 0x00, 0x00, 0x06,  // Header size
+    0x00, 0x00,              // MIDI type (0)
+    0x00, 0x01,              // Number of tracks
+    0x00, 0x46,              // Resolution
+    'M',  'T',  'r',  'k',   // Start of track
+    0x00, 0x00, 0x00, 0x00   // Placeholder for track length
 };
 
 // Cached channel velocities
@@ -97,8 +97,9 @@ static unsigned int tracksize;
 
 static const byte controller_map[] =
 {
-    0x00, 0x20, 0x01, 0x07, 0x0A, 0x0B, 0x5B, 0x5D,
-    0x40, 0x43, 0x78, 0x7B, 0x7E, 0x7F, 0x79
+    0x00, 0x20, 0x01, 0x07, 0x0A,
+    0x0B, 0x5B, 0x5D, 0x40, 0x43,
+    0x78, 0x7B, 0x7E, 0x7F, 0x79
 };
 
 static int channel_map[NUM_CHANNELS];
@@ -139,7 +140,6 @@ static boolean WriteTime(unsigned int time, MEMFILE *midioutput)
     }
 }
 
-
 // Write the end of track marker
 static boolean WriteEndTrack(MEMFILE *midioutput)
 {
@@ -160,8 +160,8 @@ static boolean WriteEndTrack(MEMFILE *midioutput)
 }
 
 // Write a key press event
-static boolean WritePressKey(byte channel, byte key,
-                             byte velocity, MEMFILE *midioutput)
+static boolean WritePressKey(byte channel, byte key, byte velocity,
+                             MEMFILE *midioutput)
 {
     byte working = midi_presskey | channel;
 
@@ -195,8 +195,7 @@ static boolean WritePressKey(byte channel, byte key,
 }
 
 // Write a key release event
-static boolean WriteReleaseKey(byte channel, byte key,
-                               MEMFILE *midioutput)
+static boolean WriteReleaseKey(byte channel, byte key, MEMFILE *midioutput)
 {
     byte working = midi_releasekey | channel;
 
@@ -230,8 +229,7 @@ static boolean WriteReleaseKey(byte channel, byte key,
 }
 
 // Write a pitch wheel/bend event
-static boolean WritePitchWheel(byte channel, short wheel,
-                               MEMFILE *midioutput)
+static boolean WritePitchWheel(byte channel, short wheel, MEMFILE *midioutput)
 {
     byte working = midi_pitchwheel | channel;
 
@@ -264,8 +262,7 @@ static boolean WritePitchWheel(byte channel, short wheel,
 }
 
 // Write a patch change event
-static boolean WriteChangePatch(byte channel, byte patch,
-                                MEMFILE *midioutput)
+static boolean WriteChangePatch(byte channel, byte patch, MEMFILE *midioutput)
 {
     byte working = midi_changepatch | channel;
 
@@ -293,10 +290,8 @@ static boolean WriteChangePatch(byte channel, byte patch,
 
 // Write a valued controller change event
 
-static boolean WriteChangeController_Valued(byte channel,
-                                            byte control,
-                                            byte value,
-                                            MEMFILE *midioutput)
+static boolean WriteChangeController_Valued(byte channel, byte control,
+                                            byte value, MEMFILE *midioutput)
 {
     byte working = midi_changecontroller | channel;
 
@@ -320,7 +315,7 @@ static boolean WriteChangeController_Valued(byte channel,
     // Quirk in vanilla DOOM? MUS controller values should be
     // 7-bit, not 8-bit.
 
-    working = value;// & 0x7F;
+    working = value; // & 0x7F;
 
     // Fix on said quirk to stop MIDI players from complaining that
     // the value is out of range:
@@ -341,12 +336,10 @@ static boolean WriteChangeController_Valued(byte channel,
 }
 
 // Write a valueless controller change event
-static boolean WriteChangeController_Valueless(byte channel,
-                                               byte control,
+static boolean WriteChangeController_Valueless(byte channel, byte control,
                                                MEMFILE *midioutput)
 {
-    return WriteChangeController_Valued(channel, control, 0,
-                                             midioutput);
+    return WriteChangeController_Valued(channel, control, 0, midioutput);
 }
 
 // Allocate a free MIDI channel.
@@ -361,7 +354,7 @@ static int AllocateMIDIChannel(void)
 
     max = -1;
 
-    for (i=0; i<NUM_CHANNELS; ++i)
+    for (i = 0; i < NUM_CHANNELS; ++i)
     {
         if (channel_map[i] > max)
         {
@@ -421,12 +414,13 @@ static boolean ReadMusHeader(MEMFILE *file, musheader *header)
 {
     boolean result;
 
-    result = mem_fread(&header->id, sizeof(byte), 4, file) == 4
-          && mem_fread(&header->scorelength, sizeof(short), 1, file) == 1
-          && mem_fread(&header->scorestart, sizeof(short), 1, file) == 1
-          && mem_fread(&header->primarychannels, sizeof(short), 1, file) == 1
-          && mem_fread(&header->secondarychannels, sizeof(short), 1, file) == 1
-          && mem_fread(&header->instrumentcount, sizeof(short), 1, file) == 1;
+    result =
+        mem_fread(&header->id, sizeof(byte), 4, file) == 4
+        && mem_fread(&header->scorelength, sizeof(short), 1, file) == 1
+        && mem_fread(&header->scorestart, sizeof(short), 1, file) == 1
+        && mem_fread(&header->primarychannels, sizeof(short), 1, file) == 1
+        && mem_fread(&header->secondarychannels, sizeof(short), 1, file) == 1
+        && mem_fread(&header->instrumentcount, sizeof(short), 1, file) == 1;
 
     if (result)
     {
@@ -440,7 +434,6 @@ static boolean ReadMusHeader(MEMFILE *file, musheader *header)
     return result;
 }
 
-
 // Read a MUS file from a stream (musinput) and output a MIDI file to
 // a stream (midioutput).
 //
@@ -456,7 +449,6 @@ boolean mus2mid(MEMFILE *musinput, MEMFILE *midioutput)
     int channel; // Channel number
     musevent event;
 
-
     // Bunch of vars read from MUS lump
     byte key;
     byte controllernumber;
@@ -475,7 +467,7 @@ boolean mus2mid(MEMFILE *musinput, MEMFILE *midioutput)
 
     // Initialise channel map to mark all channels as unused.
 
-    for (channel=0; channel<NUM_CHANNELS; ++channel)
+    for (channel = 0; channel < NUM_CHANNELS; ++channel)
     {
         channel_map[channel] = -1;
     }
@@ -489,18 +481,15 @@ boolean mus2mid(MEMFILE *musinput, MEMFILE *midioutput)
 
 #ifdef CHECK_MUS_HEADER
     // Check MUS header
-    if (musfileheader.id[0] != 'M'
-     || musfileheader.id[1] != 'U'
-     || musfileheader.id[2] != 'S'
-     || musfileheader.id[3] != 0x1A)
+    if (musfileheader.id[0] != 'M' || musfileheader.id[1] != 'U'
+        || musfileheader.id[2] != 'S' || musfileheader.id[3] != 0x1A)
     {
         return true;
     }
 #endif
 
     // Seek to where the data is held
-    if (mem_fseek(musinput, (long)musfileheader.scorestart,
-                  MEM_SEEK_SET) != 0)
+    if (mem_fseek(musinput, (long)musfileheader.scorestart, MEM_SEEK_SET) != 0)
     {
         return true;
     }
@@ -551,7 +540,9 @@ boolean mus2mid(MEMFILE *musinput, MEMFILE *midioutput)
 
                     if (key & 0x80)
                     {
-                        if (mem_fread(&channelvelocities[channel], 1, 1, musinput) != 1)
+                        if (mem_fread(&channelvelocities[channel], 1, 1,
+                                      musinput)
+                            != 1)
                         {
                             return true;
                         }
@@ -559,8 +550,8 @@ boolean mus2mid(MEMFILE *musinput, MEMFILE *midioutput)
                         channelvelocities[channel] &= 0x7F;
                     }
 
-                    if (WritePressKey(channel, key,
-                                      channelvelocities[channel], midioutput))
+                    if (WritePressKey(channel, key, channelvelocities[channel],
+                                      midioutput))
                     {
                         return true;
                     }
@@ -589,9 +580,9 @@ boolean mus2mid(MEMFILE *musinput, MEMFILE *midioutput)
                         return true;
                     }
 
-                    if (WriteChangeController_Valueless(channel,
-                                                        controller_map[controllernumber],
-                                                        midioutput))
+                    if (WriteChangeController_Valueless(
+                            channel, controller_map[controllernumber],
+                            midioutput))
                     {
                         return true;
                     }
@@ -624,10 +615,9 @@ boolean mus2mid(MEMFILE *musinput, MEMFILE *midioutput)
                             return true;
                         }
 
-                        if (WriteChangeController_Valued(channel,
-                                                         controller_map[controllernumber],
-                                                         controllervalue,
-                                                         midioutput))
+                        if (WriteChangeController_Valued(
+                                channel, controller_map[controllernumber],
+                                controllervalue, midioutput))
                         {
                             return true;
                         }
@@ -697,8 +687,8 @@ boolean mus2mid(MEMFILE *musinput, MEMFILE *midioutput)
 
 #ifdef STANDALONE
 
-#include "m_misc.h"
-#include "z_zone.h"
+#  include "m_misc.h"
+#  include "z_zone.h"
 
 int main(int argc, char *argv[])
 {
@@ -735,4 +725,3 @@ int main(int argc, char *argv[])
 }
 
 #endif
-
