@@ -41,7 +41,7 @@ extern boolean net_client_received_wait_data;
 extern net_waitdata_t net_client_wait_data;
 extern char *net_client_reject_reason;
 extern boolean net_waiting_for_launch;
-extern char *net_player_name;
+extern const char *net_player_name;
 
 extern sha1_digest_t net_server_wad_sha1sum;
 extern sha1_digest_t net_server_deh_sha1sum;
