@@ -14,7 +14,8 @@
 //  GNU General Public License for more details.
 //
 // DESCRIPTION:
-//      support maps with NODES in uncompressed XNOD/XGLN or compressed ZNOD/ZGLN formats, or DeePBSP format
+//      support maps with NODES in uncompressed XNOD/XGLN or compressed
+//      ZNOD/ZGLN formats, or DeePBSP format
 //
 //-----------------------------------------------------------------------------
 
@@ -41,7 +42,7 @@
 // [FG] support maps with NODES in DeePBSP format
 
 #if defined(_MSC_VER)
-#pragma pack(push, 1)
+#  pragma pack(push, 1)
 #endif
 
 typedef PACKED_PREFIX struct
@@ -95,10 +96,11 @@ typedef PACKED_PREFIX struct
 } PACKED_SUFFIX mapsubsector_xnod_t;
 
 #if defined(_MSC_VER)
-#pragma pack(pop)
+#  pragma pack(pop)
 #endif
 
-// [FG] support maps with NODES in uncompressed XNOD/XGLN or compressed ZNOD/ZGLN formats, or DeePBSP format
+// [FG] support maps with NODES in uncompressed XNOD/XGLN or compressed
+// ZNOD/ZGLN formats, or DeePBSP format
 
 mapformat_t P_CheckMapFormat(int lumpnum)
 {
@@ -138,17 +140,29 @@ mapformat_t P_CheckMapFormat(int lumpnum)
             lump_data = W_CacheLumpNum(lumpnum + ML_SSECTORS, PU_STATIC);
 
             if (!memcmp(lump_data, "XGLN", 4))
+            {
                 format = MFMT_XGLN;
+            }
             else if (!memcmp(lump_data, "ZGLN", 4))
+            {
                 format = MFMT_ZGLN;
+            }
             else if (!memcmp(lump_data, "XGL2", 4))
+            {
                 format = MFMT_XGL2;
+            }
             else if (!memcmp(lump_data, "ZGL2", 4))
+            {
                 format = MFMT_ZGL2;
+            }
             else if (!memcmp(lump_data, "XGL3", 4))
+            {
                 format = MFMT_XGL3;
+            }
             else if (!memcmp(lump_data, "ZGL3", 4))
+            {
                 format = MFMT_ZGL3;
+            }
         }
         else
         {
@@ -171,11 +185,17 @@ mapformat_t P_CheckMapFormat(int lumpnum)
             lump_data = W_CacheLumpNum(lumpnum + ML_NODES, PU_STATIC);
 
             if (!memcmp(lump_data, "xNd4\0\0\0\0", 8))
+            {
                 format = MFMT_DEEP;
+            }
             else if (!memcmp(lump_data, "XNOD", 4))
+            {
                 format = MFMT_XNOD;
+            }
             else if (!memcmp(lump_data, "ZNOD", 4))
+            {
                 format = MFMT_ZNOD;
+            }
         }
         else
         {
@@ -207,7 +227,7 @@ int P_GetOffset(vertex_t *v1, vertex_t *v2)
 
     a = (float)(v1->x - v2->x) / (float)FRACUNIT;
     b = (float)(v1->y - v2->y) / (float)FRACUNIT;
-    r = (int)(sqrt(a*a+b*b) * (float)FRACUNIT);
+    r = (int)(sqrt(a * a + b * b) * (float)FRACUNIT);
 
     return r;
 }
@@ -227,7 +247,7 @@ void P_LoadSegs_DEEP(int lump)
     for (i = 0; i < numsegs; i++)
     {
         seg_t *li = segs + i;
-        mapseg_deep_t *ml = (mapseg_deep_t *) data + i;
+        mapseg_deep_t *ml = (mapseg_deep_t *)data + i;
         int side, linedef;
         line_t *ldef;
         int vn1, vn2;
@@ -242,12 +262,15 @@ void P_LoadSegs_DEEP(int lump)
 
         li->angle = (SHORT(ml->angle)) << 16;
         li->offset = (SHORT(ml->offset)) << 16;
-        linedef = (unsigned short) SHORT(ml->linedef); // [FG] extended nodes
+
+        linedef = (unsigned short)SHORT(ml->linedef); // [FG] extended nodes
         ldef = &lines[linedef];
         li->linedef = ldef;
+
         side = SHORT(ml->side);
         li->sidedef = &sides[ldef->sidenum[side]];
         li->frontsector = sides[ldef->sidenum[side]].sector;
+
         // [FG] recalculate
         li->offset = P_GetOffset(li->v1, (ml->side ? ldef->v2 : ldef->v1));
 
@@ -281,14 +304,14 @@ void P_LoadSubsectors_DEEP(int lump)
 
     numsubsectors = W_LumpLength(lump) / sizeof(mapsubsector_deep_t);
     subsectors = Z_Malloc(numsubsectors * sizeof(subsector_t), PU_LEVEL, 0);
-    data = (mapsubsector_deep_t *) W_CacheLumpNum(lump, PU_STATIC);
+    data = (mapsubsector_deep_t *)W_CacheLumpNum(lump, PU_STATIC);
 
     memset(subsectors, 0, numsubsectors * sizeof(subsector_t));
 
     for (i = 0; i < numsubsectors; i++)
     {
         // [MB] 2020-04-22: Fix endianess for DeePBSDP V4 nodes
-        subsectors[i].numlines = (unsigned short) SHORT(data[i].numsegs);
+        subsectors[i].numlines = (unsigned short)SHORT(data[i].numsegs);
         subsectors[i].firstline = LONG(data[i].firstseg);
     }
 
@@ -310,7 +333,7 @@ void P_LoadNodes_DEEP(int lump)
     for (i = 0; i < numnodes; i++)
     {
         node_t *no = nodes + i;
-        mapnode_deep_t *mn = (mapnode_deep_t *) data + i;
+        mapnode_deep_t *mn = (mapnode_deep_t *)data + i;
         int j;
 
         no->x = SHORT(mn->x) << FRACBITS;
@@ -326,17 +349,20 @@ void P_LoadNodes_DEEP(int lump)
             no->children[j] = LONG(mn->children[j]);
 
             for (k = 0; k < 4; k++)
+            {
                 no->bbox[j][k] = SHORT(mn->bbox[j][k]) << FRACBITS;
+            }
         }
     }
 
     W_CacheLumpNum(lump, PU_CACHE);
 }
 
-// [FG] support maps with NODES in uncompressed XNOD/XGLN or compressed ZNOD/ZGLN formats
-// adapted from prboom-plus/src/p_setup.c:1040-1331
-// heavily modified, condensed and simplyfied
-// - removed most paranoid checks, brought more in line with Vanilla P_LoadNodes()
+// [FG] support maps with NODES in uncompressed XNOD/XGLN or compressed
+// ZNOD/ZGLN formats adapted from prboom-plus/src/p_setup.c:1040-1331 heavily
+// modified, condensed and simplyfied
+// - removed most paranoid checks, brought more in line with Vanilla
+// P_LoadNodes()
 // - removed const type punning pointers
 // - added support for compressed ZNOD/ZGLN and uncompressed XGLN formats
 
@@ -352,7 +378,7 @@ static void P_LoadSegs_XNOD(byte *data)
         unsigned int linedef;
         unsigned char side;
         seg_t *li = segs + i;
-        mapseg_xnod_t *ml = (mapseg_xnod_t *) data + i;
+        mapseg_xnod_t *ml = (mapseg_xnod_t *)data + i;
         unsigned int v1, v2;
 
         v1 = LONG(ml->v1);
@@ -360,17 +386,17 @@ static void P_LoadSegs_XNOD(byte *data)
         li->v1 = &vertexes[v1];
         li->v2 = &vertexes[v2];
 
-        linedef = (unsigned short) SHORT(ml->linedef);
+        linedef = (unsigned short)SHORT(ml->linedef);
         ldef = &lines[linedef];
         li->linedef = ldef;
         side = ml->side;
 
         // Andrey Budko: check for wrong indexes
-        if ((unsigned) ldef->sidenum[side] >= (unsigned) numsides)
+        if ((unsigned)ldef->sidenum[side] >= (unsigned)numsides)
         {
             I_Error("P_LoadSegs_XNOD: linedef %d for seg %d references a "
                     "non-existent sidedef %d",
-                    linedef, i, (unsigned) ldef->sidenum[side]);
+                    linedef, i, (unsigned)ldef->sidenum[side]);
         }
 
         li->sidedef = &sides[ldef->sidenum[side]];
@@ -407,7 +433,7 @@ static void P_LoadSegs_XNOD(byte *data)
 static void P_LoadSegs_XGLN(byte *data)
 {
     int i, j;
-    const mapseg_xnod_t *ml = (const mapseg_xnod_t *) data;
+    const mapseg_xnod_t *ml = (const mapseg_xnod_t *)data;
 
     for (i = 0; i < numsubsectors; ++i)
     {
@@ -421,11 +447,13 @@ static void P_LoadSegs_XGLN(byte *data)
 
             v1 = LONG(ml->v1);
             // partner = LONG(ml->v2);
-            line = (unsigned short) SHORT(ml->linedef);
+            line = (unsigned short)SHORT(ml->linedef);
             side = ml->side;
 
             if (line == 0xffff)
+            {
                 line = 0xffffffff;
+            }
 
             ml++;
 
@@ -445,11 +473,11 @@ static void P_LoadSegs_XGLN(byte *data)
             {
                 line_t *ldef;
 
-                if ((unsigned int) line >= (unsigned int) numlines)
+                if ((unsigned int)line >= (unsigned int)numlines)
                 {
                     I_Error("P_LoadSegs_XGLN: seg %d, %d references a "
                             "non-existent linedef %d",
-                            i, j, (unsigned int) line);
+                            i, j, (unsigned int)line);
                 }
 
                 ldef = &lines[line];
@@ -459,21 +487,21 @@ static void P_LoadSegs_XGLN(byte *data)
                 {
                     I_Error("P_LoadSegs_XGLN: seg %d, %d references a "
                             "non-existent side %d",
-                            i, j, (unsigned int) side);
+                            i, j, (unsigned int)side);
                 }
 
-                if ((unsigned) ldef->sidenum[side] >= (unsigned) numsides)
+                if ((unsigned)ldef->sidenum[side] >= (unsigned)numsides)
                 {
                     I_Error("P_LoadSegs_XGLN: linedef %d for seg %d, %d "
                             "references a non-existent sidedef %d",
-                            line, i, j, (unsigned) ldef->sidenum[side]);
+                            line, i, j, (unsigned)ldef->sidenum[side]);
                 }
 
                 seg->sidedef = &sides[ldef->sidenum[side]];
 
-                /* cph 2006/09/30 - our frontsector can be the second side of the
-                 * linedef, so must check for NO_INDEX in case we are incorrectly
-                 * referencing the back of a 1S line */
+                /* cph 2006/09/30 - our frontsector can be the second side of
+                 * the linedef, so must check for NO_INDEX in case we are
+                 * incorrectly referencing the back of a 1S line */
                 if (ldef->sidenum[side] != NO_INDEX)
                 {
                     seg->frontsector = sides[ldef->sidenum[side]].sector;
@@ -487,11 +515,15 @@ static void P_LoadSegs_XGLN(byte *data)
                         i, j);
                 }
 
-                if ((ldef->flags & ML_TWOSIDED) &&
-                    (ldef->sidenum[side ^ 1] != NO_INDEX))
+                if ((ldef->flags & ML_TWOSIDED)
+                    && (ldef->sidenum[side ^ 1] != NO_INDEX))
+                {
                     seg->backsector = sides[ldef->sidenum[side ^ 1]].sector;
+                }
                 else
+                {
                     seg->backsector = 0;
+                }
 
                 seg->offset =
                     P_GetOffset(seg->v1, (side ? ldef->v2 : ldef->v1));
@@ -515,8 +547,10 @@ static void P_LoadSegs_XGLN(byte *data)
             seg = &segs[subsectors[i].firstline + j];
 
             if (seg->linedef)
+            {
                 seg->angle = R_PointToAngle2(seg->v1->x, seg->v1->y, seg->v2->x,
                                              seg->v2->y);
+            }
         }
     }
 }
@@ -557,8 +591,10 @@ void P_LoadNodes_XNOD(int lump, boolean compressed, boolean glnodes)
         zstream->avail_out = outlen;
 
         if (inflateInit(zstream) != Z_OK)
+        {
             I_Error("P_LoadNodes_XNOD: Error during ZNOD nodes decompression "
                     "initialization!");
+        }
 
         // resize if output buffer runs full
         while ((err = inflate(zstream, Z_SYNC_FLUSH)) == Z_OK)
@@ -571,17 +607,21 @@ void P_LoadNodes_XNOD(int lump, boolean compressed, boolean glnodes)
         }
 
         if (err != Z_STREAM_END)
+        {
             I_Error("P_LoadNodes_XNOD: Error during ZNOD nodes decompression!");
+        }
 
         I_Printf(VB_DEBUG,
                  "P_LoadNodes_XNOD: ZNOD nodes compression ratio %.3f",
-                 (float) zstream->total_out / zstream->total_in);
+                 (float)zstream->total_out / zstream->total_in);
 
         data = output;
 
         if (inflateEnd(zstream) != Z_OK)
+        {
             I_Error("P_LoadNodes_XNOD: Error during ZNOD nodes decompression "
                     "shut-down!");
+        }
 
         // release the original data lump
         W_CacheLumpNum(lump, PU_CACHE);
@@ -595,13 +635,13 @@ void P_LoadNodes_XNOD(int lump, boolean compressed, boolean glnodes)
 
     // 1. Load new vertices added during node building
 
-    orgVerts = LONG(*((unsigned int *) data));
+    orgVerts = LONG(*((unsigned int *)data));
     data += sizeof(orgVerts);
 
-    newVerts = LONG(*((unsigned int *) data));
+    newVerts = LONG(*((unsigned int *)data));
     data += sizeof(newVerts);
 
-    if (orgVerts + newVerts == (unsigned int) numvertexes)
+    if (orgVerts + newVerts == (unsigned int)numvertexes)
     {
         newvertarray = vertexes;
     }
@@ -616,17 +656,17 @@ void P_LoadNodes_XNOD(int lump, boolean compressed, boolean glnodes)
     for (i = 0; i < newVerts; i++)
     {
         newvertarray[i + orgVerts].r_x = newvertarray[i + orgVerts].x =
-            LONG(*((unsigned int *) data));
+            LONG(*((unsigned int *)data));
         data += sizeof(newvertarray[0].x);
 
         newvertarray[i + orgVerts].r_y = newvertarray[i + orgVerts].y =
-            LONG(*((unsigned int *) data));
+            LONG(*((unsigned int *)data));
         data += sizeof(newvertarray[0].y);
     }
 
     if (vertexes != newvertarray)
     {
-        for (i = 0; i < (unsigned int) numlines; i++)
+        for (i = 0; i < (unsigned int)numlines; i++)
         {
             lines[i].v1 = lines[i].v1 - vertexes + newvertarray;
             lines[i].v2 = lines[i].v2 - vertexes + newvertarray;
@@ -639,18 +679,20 @@ void P_LoadNodes_XNOD(int lump, boolean compressed, boolean glnodes)
 
     // 2. Load subsectors
 
-    numSubs = LONG(*((unsigned int *) data));
+    numSubs = LONG(*((unsigned int *)data));
     data += sizeof(numSubs);
 
     if (numSubs < 1)
+    {
         I_Error("P_LoadNodes_XNOD: No subsectors in map!");
+    }
 
     numsubsectors = numSubs;
     subsectors = Z_Malloc(numsubsectors * sizeof(subsector_t), PU_LEVEL, 0);
 
     for (i = currSeg = 0; i < numsubsectors; i++)
     {
-        mapsubsector_xnod_t *mseg = (mapsubsector_xnod_t *) data + i;
+        mapsubsector_xnod_t *mseg = (mapsubsector_xnod_t *)data + i;
 
         subsectors[i].firstline = currSeg;
         subsectors[i].numlines = LONG(mseg->numsegs);
@@ -661,10 +703,11 @@ void P_LoadNodes_XNOD(int lump, boolean compressed, boolean glnodes)
 
     // 3. Load segs
 
-    numSegs = LONG(*((unsigned int *) data));
+    numSegs = LONG(*((unsigned int *)data));
     data += sizeof(numSegs);
 
-    // The number of stored segs should match the number of segs used by subsectors
+    // The number of stored segs should match the number of segs used by
+    // subsectors
     if (numSegs != currSeg)
     {
         I_Error("P_LoadNodes_XNOD: Incorrect number of segs in XNOD nodes!");
@@ -686,7 +729,7 @@ void P_LoadNodes_XNOD(int lump, boolean compressed, boolean glnodes)
 
     // 4. Load nodes
 
-    numNodes = LONG(*((unsigned int *) data));
+    numNodes = LONG(*((unsigned int *)data));
     data += sizeof(numNodes);
 
     numnodes = numNodes;
@@ -696,7 +739,7 @@ void P_LoadNodes_XNOD(int lump, boolean compressed, boolean glnodes)
     {
         int j, k;
         node_t *no = nodes + i;
-        mapnode_xnod_t *mn = (mapnode_xnod_t *) data + i;
+        mapnode_xnod_t *mn = (mapnode_xnod_t *)data + i;
 
         no->x = SHORT(mn->x) << FRACBITS;
         no->y = SHORT(mn->y) << FRACBITS;
@@ -708,12 +751,18 @@ void P_LoadNodes_XNOD(int lump, boolean compressed, boolean glnodes)
             no->children[j] = LONG(mn->children[j]);
 
             for (k = 0; k < 4; k++)
+            {
                 no->bbox[j][k] = SHORT(mn->bbox[j][k]) << FRACBITS;
+            }
         }
     }
 
     if (compressed && output)
+    {
         Z_Free(output);
+    }
     else
+    {
         W_CacheLumpNum(lump, PU_CACHE);
+    }
 }
