@@ -31,6 +31,7 @@
 #include "p_mobj.h"
 #include "p_spec.h"
 #include "p_tick.h"
+#include "r_data.h"
 #include "r_defs.h"
 #include "r_state.h"
 #include "s_sound.h"
@@ -110,8 +111,6 @@ result_e T_MovePlane
             // http://prboom.sourceforge.net/mbf-bugs.html
             if ((flag == true) && demo_compatibility)
             {
-              extern boolean P_ChangeSector(sector_t *sector,boolean crunch);
-
               sector->floorheight = lastpos;
               P_ChangeSector(sector,crush);
               return crushed;
@@ -123,7 +122,7 @@ result_e T_MovePlane
           // Moving a floor up
           // jff 02/04/98 keep floor from moving thru ceilings
           // jff 2/22/98 weaken check to demo_compatibility
-          destheight = (demo_compatibility || (demo_version >= 203 && comp[comp_floors]) ||
+          destheight = (demo_compatibility || (demo_version >= DV_MBF && comp[comp_floors]) ||
 			dest<sector->ceilingheight)? // killough 10/98
                           dest : sector->ceilingheight;
           if (sector->floorheight + speed > destheight)
@@ -146,7 +145,7 @@ result_e T_MovePlane
             flag = P_CheckSector(sector,crush); //jff 3/19/98 use faster chk
             if (flag == true)
             {
-              if (demo_compatibility || (demo_version >= 203 && comp[comp_floors])) // killough 10/98
+              if (demo_compatibility || (demo_version >= DV_MBF && comp[comp_floors])) // killough 10/98
                 if (crush == true) //jff 1/25/98 fix floor crusher
                   return crushed;
               sector->floorheight = lastpos;
@@ -223,9 +222,8 @@ result_e T_MovePlane
           }
           else
           {
-            lastpos = sector->ceilingheight;
             sector->ceilingheight += speed;
-            flag = P_CheckSector(sector,crush); //jff 3/19/98 use faster chk
+            P_CheckSector(sector,crush); //jff 3/19/98 use faster chk
           }
           break;
       }
@@ -262,7 +260,7 @@ void T_MoveFloor(floormove_t* floor)
   );
   
   if (!(leveltime&7))     // make the floormove sound
-    S_StartSound((mobj_t *)&floor->sector->soundorg, sfx_stnmov);
+    S_StartSoundPitch((mobj_t *)&floor->sector->soundorg, sfx_stnmov, PITCH_NONE);
     
   if (res == pastdest)    // if destination height is reached
   {
@@ -413,7 +411,7 @@ void T_MoveElevator(elevator_t* elevator)
 
   // make floor move sound
   if (!(leveltime&7))
-    S_StartSound((mobj_t *)&elevator->sector->soundorg, sfx_stnmov);
+    S_StartSoundPitch((mobj_t *)&elevator->sector->soundorg, sfx_stnmov, PITCH_NONE);
     
   if (res == pastdest)            // if destination height acheived
   {
@@ -752,7 +750,6 @@ int EV_BuildStairs
   int ssec = -1;
   int minssec = -1;
 
-  secnum = -1;
   rtn = 0;
 
 // [FG] Compatibility bug in EV_BuildStairs
@@ -834,7 +831,7 @@ int EV_BuildStairs
         if (tsec->floorpic != texture)
           continue;
 
-	if (comp[comp_stairs] || demo_version == 203)
+	if (comp[comp_stairs] || demo_version == DV_MBF)
 	{
 	height += stairsize;  // killough 10/98: intentionally left this way
 	}
@@ -843,7 +840,7 @@ int EV_BuildStairs
         if (P_SectorActive(floor_special,tsec)) //jff 2/22/98
 	  continue;
 
-	if (!comp[comp_stairs] && demo_version != 203)
+	if (!comp[comp_stairs] && demo_version != DV_MBF)
 	{
 	height += stairsize;
 	}
@@ -879,7 +876,7 @@ int EV_BuildStairs
     {
       // cph 2001/09/22 - emulate buggy MBF comp_stairs for demos, with logic
       // reversed since we now have a separate outer loop index.
-      if (demo_version == 203)
+      if (demo_version == DV_MBF)
         ssec = secnum; // Trash outer loop index
       else
       {
@@ -905,8 +902,6 @@ int EV_BuildStairs
 
 static boolean DonutOverrun(fixed_t *pfloorheight, short *pfloorpic)
 {
-  extern int numflats;
-
   if (demo_compatibility && overflow[emu_donut].enabled)
   {
     overflow[emu_donut].triggered = true;
