@@ -206,8 +206,8 @@ boolean P_GiveWeapon(player_t *player, weapontype_t weapon, boolean dropped)
 
       P_GiveAmmo(player, weaponinfo[weapon].ammo, deathmatch ? 5 : 2);
 
-      player->pendingweapon = weapon;
-      S_StartSound(player->mo, sfx_wpnup); // killough 4/25/98, 12/98
+      player->nextweapon = player->pendingweapon = weapon;
+      S_StartSoundPreset(player->mo, sfx_wpnup, PITCH_FULL); // killough 4/25/98, 12/98
       return false;
     }
 
@@ -216,7 +216,7 @@ boolean P_GiveWeapon(player_t *player, weapontype_t weapon, boolean dropped)
     P_GiveAmmo(player, weaponinfo[weapon].ammo, dropped ? 1 : 2);
 
   return !player->weaponowned[weapon] ?
-    player->weaponowned[player->pendingweapon = weapon] = true : gaveammo;
+    player->weaponowned[player->nextweapon = player->pendingweapon = weapon] = true : gaveammo;
 }
 
 //
@@ -478,7 +478,7 @@ void P_TouchSpecialThing(mobj_t *special, mobj_t *toucher)
       if (player->readyweapon != wp_fist)
 	if (!beta_emulation // killough 10/98: don't switch as much in -beta
 	    || player->readyweapon == wp_pistol)
-	  player->pendingweapon = wp_fist;
+	  player->nextweapon = player->pendingweapon = wp_fist;
       sound = sfx_getpow;
       break;
 
@@ -593,7 +593,7 @@ void P_TouchSpecialThing(mobj_t *special, mobj_t *toucher)
     case SPR_BFUG:
       if (!P_GiveWeapon (player, wp_bfg, false) )
         return;
-      if (classic_bfg || beta_emulation)
+      if (STRICTMODE(classic_bfg) || beta_emulation)
         pickupmsg(player, "You got the BFG2704!  Oh, yes."); // killough 8/9/98: beta BFG
       else
         pickupmsg(player, "%s", s_GOTBFG9000); // Ty 03/22/98 - externalized
@@ -652,7 +652,8 @@ void P_TouchSpecialThing(mobj_t *special, mobj_t *toucher)
   P_RemoveMobj (special);
   player->bonuscount += BONUSADD;
 
-  S_StartSound(player->mo, sound);   // killough 4/25/98, 12/98
+  S_StartSoundPreset(player->mo, sound, // killough 4/25/98, 12/98
+                     sound == sfx_itemup ? PITCH_NONE : PITCH_FULL);
 }
 
 //
@@ -727,7 +728,7 @@ static void P_KillMobj(mobj_t *source, mobj_t *target, method_t mod)
 
             if (playerscount)
             {
-              if (demo_version >= 203)
+              if (demo_version >= DV_MBF)
                 i = P_Random(pr_friends) % playerscount;
               else
                 i = Woof_Random() % playerscount;
@@ -936,7 +937,7 @@ void P_DamageMobjBy(mobj_t *target,mobj_t *inflictor, mobj_t *source, int damage
     }
 
   // killough 9/7/98: keep track of targets so that friends can help friends
-  if (demo_version >= 203)
+  if (demo_version >= DV_MBF)
     {
       // If target is a player, set player's target to source,
       // so that a friend can tell who's hurting a player
@@ -969,7 +970,7 @@ void P_DamageMobjBy(mobj_t *target,mobj_t *inflictor, mobj_t *source, int damage
   if (source && source != target && !(source->flags2 & MF2_DMGIGNORED) &&
       (!target->threshold || target->flags2 & MF2_NOTHRESHOLD) &&
       ((source->flags ^ target->flags) & MF_FRIEND || 
-       monster_infighting || demo_version < 203) &&
+       monster_infighting || demo_version < DV_MBF) &&
       !P_InfightingImmune(target, source))
     {
       // if not intent on another player, chase after this one
@@ -979,7 +980,7 @@ void P_DamageMobjBy(mobj_t *target,mobj_t *inflictor, mobj_t *source, int damage
       // killough 9/9/98: cleaned up, made more consistent:
 
       if (!target->lastenemy || target->lastenemy->health <= 0 ||
-	  (demo_version < 203 ? !target->lastenemy->player :
+	  (demo_version < DV_MBF ? !target->lastenemy->player :
 	   !((target->flags ^ target->lastenemy->flags) & MF_FRIEND) &&
 	   target->target != source)) // remember last enemy - killough
 	P_SetTarget(&target->lastenemy, target->target);
