@@ -419,9 +419,6 @@ extern int itemrespawntime[];
 extern int iquehead;
 extern int iquetail;
 
-// [FG] colored blood and gibs
-extern boolean colored_blood;
-
 extern boolean direct_vertical_aiming, default_direct_vertical_aiming;
 void P_UpdateDirectVerticalAiming(void);
 
@@ -436,7 +433,9 @@ boolean P_SetMobjState(mobj_t *mobj, statenum_t state);
 void    P_MobjThinker(mobj_t *mobj);
 void    P_SpawnPuff(fixed_t x, fixed_t y, fixed_t z);
 void    P_SpawnBlood(fixed_t x, fixed_t y, fixed_t z, int damage, mobj_t *bleeder);
+void    P_SpawnPlayer (mapthing_t* mthing);
 mobj_t  *P_SpawnMissile(mobj_t *source, mobj_t *dest, mobjtype_t type);
+extern int autoaim;
 mobj_t  *P_SpawnPlayerMissile(mobj_t *source, mobjtype_t type);
 void    P_SpawnMapThing (mapthing_t*  mthing);
 boolean P_CheckMissileSpawn(mobj_t*);  // killough 8/2/98
