@@ -27,6 +27,7 @@
 #include "doomdata.h"
 #include "doomstat.h"
 #include "g_game.h"
+#include "g_compatibility.h"
 #include "i_printf.h"
 #include "i_system.h"
 #include "info.h"
@@ -49,7 +50,6 @@
 #include "r_things.h"
 #include "s_musinfo.h" // [crispy] S_ParseMusInfo()
 #include "s_sound.h"
-#include "st_stuff.h"
 #include "tables.h"
 #include "w_wad.h"
 #include "z_zone.h"
@@ -357,7 +357,8 @@ void P_LoadSectors (int lump)
       // [FG] inhibit sector interpolation during the 0th gametic
       ss->oldceilgametic = -1;
       ss->oldfloorgametic = -1;
-      ss->oldscrollgametic = -1;
+      ss->old_ceil_offs_gametic = -1;
+      ss->old_floor_offs_gametic = -1;
     }
 
   Z_Free (data);
@@ -1197,13 +1198,13 @@ boolean P_LoadBlockMap (int lump)
 
       blockmaplump[0] = SHORT(wadblockmaplump[0]);
       blockmaplump[1] = SHORT(wadblockmaplump[1]);
-      blockmaplump[2] = (long)(SHORT(wadblockmaplump[2])) & 0xffff;
-      blockmaplump[3] = (long)(SHORT(wadblockmaplump[3])) & 0xffff;
+      blockmaplump[2] = (long)(SHORT(wadblockmaplump[2])) & FRACMASK;
+      blockmaplump[3] = (long)(SHORT(wadblockmaplump[3])) & FRACMASK;
 
       for (i=4 ; i<count ; i++)
         {
           short t = SHORT(wadblockmaplump[i]);          // killough 3/1/98
-          blockmaplump[i] = t == -1 ? -1l : (long) t & 0xffff;
+          blockmaplump[i] = t == -1 ? -1l : (long) t & FRACMASK;
         }
 
       Z_Free(wadblockmaplump);
@@ -1296,10 +1297,10 @@ int P_GroupLines (void)
       sector->lines -= sector->linecount;
 
       // set the degenmobj_t to the middle of the bounding box
-      sector->soundorg.x = (sector->blockbox[BOXRIGHT] + 
-			    sector->blockbox[BOXLEFT])/2;
-      sector->soundorg.y = (sector->blockbox[BOXTOP] + 
-			    sector->blockbox[BOXBOTTOM])/2;
+      sector->soundorg.x =
+          sector->blockbox[BOXRIGHT] / 2 + sector->blockbox[BOXLEFT] / 2;
+      sector->soundorg.y =
+          sector->blockbox[BOXTOP] / 2 + sector->blockbox[BOXBOTTOM] / 2;
 
       sector->soundorg.thinker.function.p1 = (actionf_p1)P_DegenMobjThinker;
 
@@ -1399,7 +1400,7 @@ void P_RemoveSlimeTrails(void)                // killough 10/98
 		    v->r_y = (fixed_t)((dy2 * y0 + dx2 * y1 + dxy * (x0 - x1)) / s);
 
 		    // [FG] override actual vertex coordinates except in compatibility mode
-		    if (demo_version >= 203)
+		    if (demo_version >= DV_MBF)
 		    {
 		      v->x = v->r_x;
 		      v->y = v->r_y;
@@ -1603,10 +1604,12 @@ void P_SetupLevel(int episode, int map, int playermask, skill_t skill)
   //    W_Reload ();     killough 1/31/98: W_Reload obsolete
 
   // find map name
-  strcpy(lumpname, MAPNAME(episode, map));
+  strcpy(lumpname, MapName(episode, map));
 
   lumpnum = W_GetNumForName(lumpname);
 
+  G_ApplyLevelCompatibility(lumpnum);
+
   leveltime = 0;
   oldleveltime = 0;
 
@@ -1661,8 +1664,6 @@ void P_SetupLevel(int episode, int map, int playermask, skill_t skill)
 
   // [crispy] fix long wall wobble
   P_SegLengths(false);
-  // [crispy] blinking key or skull in the status bar
-  memset(st_keyorskull, 0, sizeof(st_keyorskull));
 
   // Note: you don't need to clear player queue slots --
   // a much simpler fix is in g_game.c -- killough 10/98
@@ -1703,7 +1704,7 @@ void P_SetupLevel(int episode, int map, int playermask, skill_t skill)
     R_PrecacheLevel();
 
   // [FG] log level setup
-  I_Printf(VB_INFO, "P_SetupLevel: %.8s (%s), Skill %d, %s%s%s, %s",
+  I_Printf(VB_DEMO, "P_SetupLevel: %.8s (%s), Skill %d, %s%s%s, %s",
     lumpname, W_WadNameForLump(lumpnum),
     gameskill + 1,
     mapformat >= MFMT_UNSUPPORTED ? "NanoBSP" :
