@@ -28,9 +28,9 @@
 #include "doomstat.h"
 #include "doomtype.h"
 #include "g_game.h"
-#include "hu_stuff.h"
+#include "g_nextweapon.h"
 #include "info.h"
-#include "m_input.h"
+#include "m_cheat.h"
 #include "p_map.h"
 #include "p_mobj.h"
 #include "p_pspr.h"
@@ -38,21 +38,13 @@
 #include "p_user.h"
 #include "r_defs.h"
 #include "r_main.h"
-#include "r_state.h"
+#include "st_stuff.h"
+#include "st_widgets.h"
 
 static fixed_t PlayerSlope(player_t *player)
 {
   const fixed_t pitch = player->pitch;
-
-  if (pitch)
-  {
-    const fixed_t slope = -finetangent[(ANG90 - pitch) >> ANGLETOFINESHIFT];
-    return (fixed_t)((int64_t)slope * SCREENHEIGHT / ACTUALHEIGHT);
-  }
-  else
-  {
-    return 0;
-  }
+  return pitch ? -finetangent[(ANG90 - pitch) >> ANGLETOFINESHIFT] : 0;
 }
 
 // Index of the special effects (INVUL inverse) map.
@@ -93,7 +85,7 @@ void P_Thrust(player_t* player,angle_t angle,fixed_t move)
 
 void P_Bob(player_t *player, angle_t angle, fixed_t move)
 {
-  if (demo_version < 203)
+  if (demo_version < DV_MBF)
     return;
 
   player->momx += FixedMul(move,finecosine[angle >>= ANGLETOFINESHIFT]);
@@ -125,14 +117,14 @@ void P_CalcHeight (player_t* player)
 
   // [FG] MBF player bobbing rewrite causes demo sync problems
   // http://prboom.sourceforge.net/mbf-bugs.html
-  player->bob = (demo_version >= 203 && player_bobbing) ?
+  player->bob = (demo_version >= DV_MBF && player_bobbing) ?
       (FixedMul(player->momx,player->momx)
       + FixedMul(player->momy,player->momy))>>2 :
       (demo_compatibility || player_bobbing) ?
       (FixedMul (player->mo->momx, player->mo->momx)
       + FixedMul (player->mo->momy,player->mo->momy))>>2 : 0;
 
-  if ((demo_version == 202 || demo_version == 203) &&
+  if ((demo_version == DV_BOOM || demo_version == DV_MBF) &&
       player->mo->friction > ORIG_FRICTION) // ice?
   {
     if (player->bob > (MAXBOB>>2))
@@ -213,12 +205,7 @@ void P_MovePlayer (player_t* player)
 
   mo->angle += cmd->angleturn << 16;
   onground = mo->z <= mo->floorz;
-
-  if (player == &players[consoleplayer])
-  {
-    localview.ticangle += localview.ticangleturn << 16;
-    localview.ticangleturn = 0;
-  }
+  player->ticangle += cmd->ticangleturn << FRACBITS;
 
   // killough 10/98:
   //
@@ -227,7 +214,7 @@ void P_MovePlayer (player_t* player)
   // ice, because the player still "works just as hard" to move, while the
   // thrust applied to the movement varies with 'movefactor'.
 
-  if ((!demo_compatibility && demo_version < 203) ||
+  if ((!demo_compatibility && demo_version < DV_MBF) ||
       cmd->forwardmove | cmd->sidemove) // killough 10/98
     {
       if (onground || mo->flags & MF_BOUNCES) // killough 8/9/98
@@ -260,9 +247,9 @@ void P_MovePlayer (player_t* player)
 	P_SetMobjState(mo,S_PLAY_RUN1);
     }
 
-  if (!menuactive && !demoplayback)
+  if (!menuactive && !demoplayback && !player->centering)
   {
-    player->pitch += cmd->pitch;
+    player->pitch += cmd->pitch << FRACBITS;
     player->pitch = BETWEEN(-MAX_PITCH_ANGLE, MAX_PITCH_ANGLE, player->pitch);
     player->slope = PlayerSlope(player);
   }
@@ -270,14 +257,8 @@ void P_MovePlayer (player_t* player)
 
 #define ANG5 (ANG90/18)
 
-typedef enum
-{
-  death_use_default,
-  death_use_reload,
-  death_use_nothing
-} death_use_action_t;
-
 death_use_action_t death_use_action;
+death_use_state_t death_use_state;
 
 //
 // P_DeathThink
@@ -335,28 +316,25 @@ void P_DeathThink (player_t* player)
 
   if (player->cmd.buttons & BT_USE)
   {
-    if (demorecording || demoplayback || netgame)
-      player->playerstate = PST_REBORN;
-    else switch(death_use_action)
+    player->playerstate = PST_REBORN;
+  }
+
+  if (death_use_state == DEATH_USE_STATE_PENDING)
+  {
+    death_use_state = DEATH_USE_STATE_ACTIVE;
+
+    if (!G_AutoSaveEnabled() || !G_LoadAutoSaveDeathUse())
     {
-      case death_use_default:
+      if (savegameslot >= 0)
+      {
+        char *file = G_SaveGameName(savegameslot);
+        G_LoadGame(file, savegameslot, false);
+        free(file);
+      }
+      else
+      {
         player->playerstate = PST_REBORN;
-        break;
-      case death_use_reload:
-        if (savegameslot >= 0)
-        {
-          char *file = G_SaveGameName(savegameslot);
-          G_LoadGame(file, savegameslot, false);
-          free(file);
-          // [Woof!] prevent on-death-action reloads from activating specials
-          M_InputGameDeactivate(input_use);
-        }
-        else
-          player->playerstate = PST_REBORN;
-        break;
-      case death_use_nothing:
-      default:
-        break;
+      }
     }
   }
 }
@@ -369,7 +347,6 @@ void P_PlayerThink (player_t* player)
 {
   ticcmd_t*    cmd;
   weapontype_t newweapon;
-  extern boolean palette_changes;
 
   // [AM] Assume we can interpolate at the beginning
   //      of the tic.
@@ -383,11 +360,7 @@ void P_PlayerThink (player_t* player)
   player->oldviewz = player->viewz;
   player->oldpitch = player->pitch;
   player->oldrecoilpitch = player->recoilpitch;
-
-  if (player == &players[consoleplayer])
-  {
-    localview.oldticangle = localview.ticangle;
-  }
+  player->oldticangle = player->ticangle;
 
   // killough 2/8/98, 3/21/98:
   // (this code is necessary despite questions raised elsewhere in a comment)
@@ -403,6 +376,7 @@ void P_PlayerThink (player_t* player)
   if (player->mo->flags & MF_JUSTATTACKED)
     {
       cmd->angleturn = 0;
+      cmd->ticangleturn = 0;
       cmd->forwardmove = 0xc800/512;
       cmd->sidemove = 0;
       player->mo->flags &= ~MF_JUSTATTACKED;
@@ -505,11 +479,12 @@ void P_PlayerThink (player_t* player)
 
       if (demo_compatibility)
 	{ // compatibility mode -- required for old demos -- killough
+	  newweapon = (cmd->buttons & BT_WEAPONMASK_OLD) >> BT_WEAPONSHIFT;
 	  if (newweapon == wp_fist && player->weaponowned[wp_chainsaw] &&
 	      (player->readyweapon != wp_chainsaw ||
 	       !player->powers[pw_strength]))
 	    newweapon = wp_chainsaw;
-	  if (have_ssg &&
+	  if (ALLOW_SSG &&
 	      newweapon == wp_shotgun &&
 	      player->weaponowned[wp_supershotgun] &&
 	      player->readyweapon != wp_supershotgun)
@@ -525,7 +500,7 @@ void P_PlayerThink (player_t* player)
 
 	if ((newweapon != wp_plasma && newweapon != wp_bfg)
 	    || (gamemode != shareware) )
-	  player->pendingweapon = newweapon;
+	  player->nextweapon = player->pendingweapon = newweapon;
     }
 
   // check for use
@@ -577,7 +552,6 @@ void P_PlayerThink (player_t* player)
 
   if (player->cheats & CF_MAPCOORDS)
   {
-    extern void cheat_mypos_print();
     cheat_mypos_print();
   }
 
@@ -618,6 +592,58 @@ void P_PlayerThink (player_t* player)
     player->powers[pw_infrared] > 4*32 || player->powers[pw_infrared] & 8;
 }
 
+boolean P_EvaluateItemOwned(itemtype_t item, player_t *player)
+{
+    switch (item)
+    {
+        case item_bluecard:
+        case item_yellowcard:
+        case item_redcard:
+        case item_blueskull:
+        case item_yellowskull:
+        case item_redskull:
+            return player->cards[item - item_bluecard] != 0;
+
+        case item_backpack:
+            return player->backpack;
+
+        case item_greenarmor:
+        case item_bluearmor:
+            return player->armortype == (item - item_greenarmor + 1);
+
+        case item_areamap:
+            return player->powers[pw_allmap] != 0;
+
+        case item_lightamp:
+            return player->powers[pw_infrared] != 0;
+
+        case item_berserk:
+            return player->powers[pw_strength] != 0;
+
+        case item_invisibility:
+            return player->powers[pw_invisibility] != 0;
+
+        case item_radsuit:
+            return player->powers[pw_ironfeet] != 0;
+
+        case item_invulnerability:
+            return player->powers[pw_invulnerability] != 0;
+
+        case item_healthbonus:
+        case item_stimpack:
+        case item_medikit:
+        case item_soulsphere:
+        case item_megasphere:
+        case item_armorbonus:
+        case item_noitem:
+        case item_messageonly:
+        default:
+            break;
+    }
+
+    return false;
+}
+
 //----------------------------------------------------------------------------
 //
 // $Log: p_user.c,v $
