@@ -22,6 +22,7 @@
 #ifndef __P_USER__
 #define __P_USER__
 
+#include "doomdef.h"
 #include "m_fixed.h"
 #include "tables.h"
 
@@ -33,6 +34,28 @@ void P_DeathThink(struct player_s *player);
 void P_MovePlayer(struct player_s *player);
 void P_Thrust(struct player_s *player, angle_t angle, fixed_t move);
 
+typedef enum
+{
+  DEATH_USE_ACTION_DEFAULT,
+  DEATH_USE_ACTION_LAST_SAVE,
+  DEATH_USE_ACTION_NOTHING
+} death_use_action_t;
+
+extern death_use_action_t death_use_action;
+
+typedef enum
+{
+  DEATH_USE_STATE_INACTIVE,
+  DEATH_USE_STATE_PENDING,
+  DEATH_USE_STATE_ACTIVE
+} death_use_state_t;
+
+extern death_use_state_t death_use_state;
+
+boolean P_EvaluateItemOwned(itemtype_t item, struct player_s *player);
+
+extern boolean onground; // whether player is on ground or in air
+
 #endif // __P_USER__
 
 //----------------------------------------------------------------------------
