@@ -37,8 +37,10 @@ typedef struct
     double mid;
     fixed_t scrollx;
     fixed_t currx;
+    fixed_t prevx;
     fixed_t scrolly;
     fixed_t curry;
+    fixed_t prevy;
     fixed_t scalex;
     fixed_t scaley;
 } skytex_t;
