@@ -22,6 +22,12 @@
 
 #include "doomtype.h"
 
+typedef enum {
+  PITCH_FULL,
+  PITCH_HALF,
+  PITCH_NONE
+} pitchrange_t;
+
 struct mobj_s;
 
 //
@@ -38,11 +44,45 @@ void S_Init(int sfxVolume, int musicVolume);
 //
 void S_Start(void);
 
+void S_StopChannels(void);
+
 //
 // Start sound for thing at <origin>
 //  using <sound_id> from sounds.h
 //
-void S_StartSound(const struct mobj_s *origin, int sound_id);
+
+// Thing at <origin> emits sound. No rumble.
+#define S_StartSound(o, i) S_StartSoundPitch((o), (i), PITCH_FULL)
+void S_StartSoundPitch(const struct mobj_s *origin, int sfx_id,
+                       pitchrange_t pitch_range);
+
+// Thing at <origin> emits sound. Rumbles if displayplayer is <origin>.
+#define S_StartSoundEx(o, i) S_StartSoundPitchEx((o), (i), PITCH_FULL)
+void S_StartSoundPitchEx(const struct mobj_s *origin, int sfx_id,
+                         pitchrange_t pitch_range);
+void S_StartSoundPistol(const struct mobj_s *origin, int sfx_id);
+void S_StartSoundShotgun(const struct mobj_s *origin, int sfx_id);
+void S_StartSoundSSG(const struct mobj_s *origin, int sfx_id);
+void S_StartSoundCGun(const struct mobj_s *origin, int sfx_id);
+void S_StartSoundBFG(const struct mobj_s *origin, int sfx_id);
+
+// Thing at <origin> emits sound. Rumbles preset if displayplayer is <origin>.
+void S_StartSoundPreset(const struct mobj_s *origin, int sfx_id,
+                        pitchrange_t pitch_range);
+void S_StartSoundPain(const struct mobj_s *origin, int sfx_id);
+void S_StartSoundHitFloor(const struct mobj_s *origin, int sfx_id);
+
+// Thing at <source> causes sound. Thing at <origin> emits sound.
+// Rumbles if displayplayer is <source>.
+void S_StartSoundSource(const struct mobj_s *source,
+                        const struct mobj_s *origin, int sfx_id);
+void S_StartSoundMissile(const struct mobj_s *source,
+                         const struct mobj_s *origin, int sfx_id);
+
+// Thing at <source> causes sound. Thing at <origin> emits sound.
+// Rumbles if displayplayer is <source>, based on distance to <origin>.
+void S_StartSoundOrigin(const struct mobj_s *source,
+                        const struct mobj_s *origin, int sfx_id);
 
 // Stop sound for thing at <origin>
 void S_StopSound(const struct mobj_s *origin);
@@ -68,8 +108,6 @@ void S_ResumeSound(void);
 
 void S_RestartMusic(void);
 
-void S_UpdateMusic(void);
-
 //
 // Updates music & sounds
 //
@@ -79,12 +117,13 @@ void S_SetMusicVolume(int volume);
 void S_SetSfxVolume(int volume);
 
 // machine-independent sound params
-extern int numChannels;
-extern int default_numChannels; // killough 10/98
+extern int snd_channels;
 
 // jff 3/17/98 holds last IDMUS number, or -1
 extern int idmusnum;
 
+void S_BindSoundVariables(void);
+
 #endif
 
 //----------------------------------------------------------------------------
