@@ -20,6 +20,7 @@
 //
 //-----------------------------------------------------------------------------
 
+#include <math.h>
 #include <stdlib.h>
 
 #include "am_map.h"
@@ -126,9 +127,6 @@ static boolean hud_armor_type; // color of armor depends on type
 
 static boolean weapon_carousel;
 
-// used for evil grin
-static boolean  oldweaponsowned[NUMWEAPONS];
-
 // [crispy] blinking key or skull in the status bar
 int st_keyorskull[3];
 
@@ -590,10 +588,13 @@ static void UpdateFace(sbe_face_t *face, player_t *player)
 
             for (int i = 0; i < NUMWEAPONS; ++i)
             {
-                if (oldweaponsowned[i] != player->weaponowned[i])
+                if (face->oldweaponsowned[i] != player->weaponowned[i])
                 {
-                    doevilgrin = true;
-                    oldweaponsowned[i] = player->weaponowned[i];
+                    if (face->oldweaponsowned[i] < player->weaponowned[i])
+                    {
+                        doevilgrin = true;
+                    }
+                    face->oldweaponsowned[i] = player->weaponowned[i];
                 }
             }
 
@@ -1047,7 +1048,7 @@ static void UpdateStatusBar(player_t *player)
     }
 }
 
-static void ResetElem(sbarelem_t *elem)
+static void ResetElem(sbarelem_t *elem, player_t *player)
 {
     switch (elem->type)
     {
@@ -1064,6 +1065,10 @@ static void ResetElem(sbarelem_t *elem)
                 face->faceindex = 0;
                 face->facecount = 0;
                 face->oldhealth = -1;
+                for (int i = 0; i < NUMWEAPONS; i++)
+                {
+                    face->oldweaponsowned[i] = player->weaponowned[i];
+                }
             }
             break;
 
@@ -1096,19 +1101,21 @@ static void ResetElem(sbarelem_t *elem)
     sbarelem_t *child;
     array_foreach(child, elem->children)
     {
-        ResetElem(child);
+        ResetElem(child, player);
     }
 }
 
 static void ResetStatusBar(void)
 {
+    player_t *player = &players[displayplayer];
+
     statusbar_t *local_statusbar;
     array_foreach(local_statusbar, sbardef->statusbars)
     {
         sbarelem_t *child;
         array_foreach(child, local_statusbar->children)
         {
-            ResetElem(child);
+            ResetElem(child, player);
         }
     }
 
