@@ -17,20 +17,19 @@
 //
 //-----------------------------------------------------------------------------
 
-#include <errno.h>
-#include <ctype.h>
-#include <fcntl.h>
 #include <stdlib.h>
 #include <string.h>
 
-#include "d_main.h" // [FG] wadfiles
+#include "config.h"
+#include "doomdef.h"
+#include "doomstat.h"
+#include "doomtype.h"
 #include "i_printf.h"
 #include "i_system.h"
 #include "m_array.h"
-#include "m_io.h"
 #include "m_misc.h"
-#include "m_swap.h"
 #include "w_wad.h"
+#include "w_internal.h"
 #include "z_zone.h"
 
 //
@@ -38,19 +37,13 @@
 //
 
 // Location of each lump on disk.
-lumpinfo_t *lumpinfo;
-int        numlumps;         // killough
-void       **lumpcache;      // killough
+lumpinfo_t  *lumpinfo = NULL;
+int         numlumps;         // killough
+void        **lumpcache;      // killough
 
-static int W_FileLength(int handle)
-{
-   struct stat fileinfo;
-   if(fstat(handle,&fileinfo) == -1)
-      I_Error("W_FileLength: failure in fstat\n");
-   return fileinfo.st_size;
-}
+const char  **wadfiles;
 
-void ExtractFileBase(const char *path, char *dest)
+void W_ExtractFileBase(const char *path, char *dest)
 {
   const char *src;
   int length;
@@ -69,134 +62,156 @@ void ExtractFileBase(const char *path, char *dest)
       break;
     }
     else
-      *dest++ = toupper(*src++);
+      *dest++ = M_ToUpper(*src++);
 }
 
 //
 // LUMP BASED ROUTINES.
 //
 
-//
-// W_AddFile
-// All files are optional, but at least one file must be
-//  found (PWAD, if all required lumps are present).
-// Files with a .wad extension are wadlink files
-//  with multiple lumps.
-// Other files are single lumps with the base filename
-//  for the lump name.
-//
-// Reload hack removed by Lee Killough
-//
-
-static int *handles = NULL;
-
-static void W_AddFile(const char *name) // killough 1/31/98: static, const
+void W_AddMarker(const char *name)
 {
-    wadinfo_t header;
-    filelump_t *fileinfo;
-    int handle;
-    int length;
-    int startlump;
+    lumpinfo_t marker = {0};
+    M_CopyLumpName(marker.name, name);
+    array_push(lumpinfo, marker);
+    numlumps++;
+}
 
-    // open the file and add to directory
+boolean W_SkipFile(const char *filename)
+{
+    static const char *ext[] = { ".wad", ".zip", ".pk3", ".deh", ".exe",
+                                 ".bat", ".txt" };
 
-    handle = M_open(name, O_RDONLY | O_BINARY);
-
-    if (handle == -1)
+    for (int i = 0; i < arrlen(ext); ++i)
     {
-        if (M_StringCaseEndsWith(name, ".lmp"))
+        if (M_StringCaseEndsWith(filename, ext[i]))
         {
-            return;
+            return true;
         }
-        I_Error("Error: couldn't open %s", name); // killough
+    }
+    return false;
+}
+
+static struct
+{
+    const char *dir;
+    const char *start_marker;
+    const char *end_marker;
+    namespace_t namespace;
+} subdirs[] = {
+    {"music",     NULL,       NULL,     ns_global   },
+    {"graphics",  NULL,       NULL,     ns_global   },
+    {"sprites",   "S_START",  "S_END",  ns_sprites  },
+    {"flats",     "F_START",  "F_END",  ns_flats    },
+    {"colormaps", "C_START",  "C_END",  ns_colormaps},
+    {"voxels",    "VX_START", "VX_END", ns_voxels   },
+};
+
+static struct
+{
+    const char *dir;
+    GameMode_t mode;
+    GameMission_t mission;
+} filters[] = {
+    {"doom.id.doom1",            shareware,    doom     },
+    {"doom.id.doom1.registered", registered,   doom     },
+    {"doom.id.doom1.ultimate",   retail,       doom     },
+    {"doom.id.doom2.commercial", commercial,   doom2    },
+    {"doom.id.doom2.plutonia",   commercial,   pack_plut},
+    {"doom.id.doom2.tnt",        commercial,   pack_tnt },
+};
+
+static w_module_t *modules[] =
+{
+    &w_zip_module,
+    &w_file_module,
+};
+
+static void AddDirs(w_module_t *module, w_handle_t handle, const char *base)
+{
+    if (!module->AddDir(handle, base, NULL, NULL))
+    {
+        return;
     }
 
-    I_Printf(VB_INFO, " adding %s", name); // killough 8/8/98
-
-    startlump = numlumps;
-
-    boolean is_single = false;
-
-    // killough:
-    if (!M_StringCaseEndsWith(name, ".wad"))
+    for (int i = 0; i < arrlen(subdirs); ++i)
     {
-        // single lump file
-        fileinfo = calloc(1, sizeof(*fileinfo));
-        fileinfo[0].size = LONG(W_FileLength(handle));
-        ExtractFileBase(name, fileinfo[0].name);
-        numlumps++;
-        is_single = true;
+        if (base[0] == '.')
+        {
+            module->AddDir(handle, subdirs[i].dir, subdirs[i].start_marker,
+                           subdirs[i].end_marker);
+        }
+        else
+        {
+            char *s = M_StringJoin(base, DIR_SEPARATOR_S, subdirs[i].dir);
+            module->AddDir(handle, s, subdirs[i].start_marker,
+                           subdirs[i].end_marker);
+            free(s);
+        }
     }
-    else
+}
+
+boolean W_AddPath(const char *path)
+{
+    static int priority;
+
+    w_handle_t handle = {0};
+    handle.priority = priority++;
+
+    w_module_t *active_module = NULL;
+
+    for (int i = 0; i < arrlen(modules); ++i)
     {
-        // WAD file
-        if (read(handle, &header, sizeof(header)) == 0)
-        {
-            I_Error("Error reading header from %s (%s)", name, strerror(errno));
-        }
+        w_type_t result = modules[i]->Open(path, &handle);
 
-        if (strncmp(header.identification, "IWAD", 4)
-            && strncmp(header.identification, "PWAD", 4))
+        if (result == W_FILE)
         {
-            I_Error("Wad file %s doesn't have IWAD or PWAD id", name);
+            return true;
         }
-
-        header.numlumps = LONG(header.numlumps);
-        if (header.numlumps == 0)
+        else if (result == W_DIR)
         {
-            I_Printf(VB_WARNING, "Wad file %s is empty", name);
-            close(handle);
-            return;
+            active_module = modules[i];
+            break;
         }
-
-        length = header.numlumps * sizeof(filelump_t);
-        fileinfo = malloc(length);
-        if (fileinfo == NULL)
-        {
-            I_Error("Failed to allocate file table from %s", name);
-        }
-
-        header.infotableofs = LONG(header.infotableofs);
-        if (lseek(handle, header.infotableofs, SEEK_SET) == -1)
-        {
-            I_Printf(VB_WARNING, "Error seeking offset from %s (%s)", name,
-                     strerror(errno));
-            close(handle);
-            free(fileinfo);
-            return;
-        }
-
-        if (read(handle, fileinfo, length) == 0)
-        {
-            I_Printf(VB_WARNING, "Error reading lump directory from %s (%s)",
-                     name, strerror(errno));
-            close(handle);
-            free(fileinfo);
-            return;
-        }
-
-        numlumps += header.numlumps;
     }
 
-    array_push(handles, handle);
-
-    // Fill in lumpinfo
-    lumpinfo = Z_Realloc(lumpinfo, numlumps * sizeof(lumpinfo_t), PU_STATIC, 0);
-
-    for (int i = startlump, j = 0; i < numlumps; i++, j++)
+    if (!active_module)
     {
-        lumpinfo[i].handle = handle; //  killough 4/25/98
-        lumpinfo[i].position = LONG(fileinfo[j].filepos);
-        lumpinfo[i].size = LONG(fileinfo[j].size);
-        lumpinfo[i].data = NULL;           // killough 1/31/98
-        lumpinfo[i].namespace = ns_global; // killough 4/17/98
-        M_CopyLumpName(lumpinfo[i].name, fileinfo[j].name);
-
-        // [FG] WAD file that contains the lump
-        lumpinfo[i].wad_file = (is_single ? NULL : name);
+        return false;
     }
 
-    free(fileinfo);
+    AddDirs(active_module, handle, ".");
+
+    char *dir = NULL;
+
+    for (int i = 0; i < arrlen(filters); ++i)
+    {
+        if (filters[i].mode == gamemode && filters[i].mission == gamemission)
+        {
+            dir = M_StringJoin("filter", DIR_SEPARATOR_S, filters[i].dir);
+            break;
+        }
+    }
+
+    if (!dir)
+    {
+        return true;
+    }
+
+    for (char *p = dir; *p; ++p)
+    {
+        if (*p == '.')
+        {
+            *p = '\0';
+            AddDirs(active_module, handle, dir);
+            *p = '.';
+        }
+    }
+    AddDirs(active_module, handle, dir);
+
+    free(dir);
+
+    return true;
 }
 
 // jff 1/23/98 Create routines to reorder the master directory
@@ -279,14 +294,14 @@ static void W_CoalesceMarkedResource(const char *start_marker,
 unsigned W_LumpNameHash(const char *s)
 {
   unsigned hash;
-  (void) ((hash =        toupper(s[0]), s[1]) &&
-          (hash = hash*3+toupper(s[1]), s[2]) &&
-          (hash = hash*2+toupper(s[2]), s[3]) &&
-          (hash = hash*2+toupper(s[3]), s[4]) &&
-          (hash = hash*2+toupper(s[4]), s[5]) &&
-          (hash = hash*2+toupper(s[5]), s[6]) &&
-          (hash = hash*2+toupper(s[6]),
-           hash = hash*2+toupper(s[7]))
+  (void) ((hash =        M_ToUpper(s[0]), s[1]) &&
+          (hash = hash*3+M_ToUpper(s[1]), s[2]) &&
+          (hash = hash*2+M_ToUpper(s[2]), s[3]) &&
+          (hash = hash*2+M_ToUpper(s[3]), s[4]) &&
+          (hash = hash*2+M_ToUpper(s[4]), s[5]) &&
+          (hash = hash*2+M_ToUpper(s[5]), s[6]) &&
+          (hash = hash*2+M_ToUpper(s[6]),
+           hash = hash*2+M_ToUpper(s[7]))
          );
   return hash;
 }
@@ -384,36 +399,50 @@ int W_GetNumForName (const char* name)     // killough -- const added
 //  does override all earlier ones.
 //
 
+static w_handle_t base_handle;
+
+boolean W_InitBaseFile(const char *path)
+{
+    char *filename =
+        M_StringJoin(path, DIR_SEPARATOR_S, PROJECT_SHORTNAME ".pk3");
+
+    w_type_t result = w_zip_module.Open(filename, &base_handle);
+
+    free(filename);
+
+    if (result == W_DIR)
+    {
+        AddDirs(&w_zip_module, base_handle, "all-all");
+        return true;
+    }
+
+    return false;
+}
+
+void W_AddBaseDir(const char *path)
+{
+    AddDirs(&w_zip_module, base_handle, path);
+}
+
 void W_InitMultipleFiles(void)
 {
-  int i;
-
-  // killough 1/31/98: add predefined lumps first
-
-  numlumps = num_predefined_lumps;
-
-  // lumpinfo will be realloced as lumps are added
-  lumpinfo = Z_Malloc(numlumps*sizeof(*lumpinfo), PU_STATIC, 0);
-
-  memcpy(lumpinfo, predefined_lumps, numlumps*sizeof(*lumpinfo));
-
-  // open all the files, load headers, and count lumps
-  for (i = 0; i < array_size(wadfiles); ++i)
-    W_AddFile(wadfiles[i]);
-
   if (!numlumps)
     I_Error ("W_InitFiles: no files found");
 
   //jff 1/23/98
   // get all the sprites and flats into one marked block each
   // killough 1/24/98: change interface to use M_START/M_END explicitly
+  // killough 4/4/98: add colormap markers
   // killough 4/17/98: Add namespace tags to each entry
 
-  W_CoalesceMarkedResource("S_START", "S_END", ns_sprites);
-  W_CoalesceMarkedResource("F_START", "F_END", ns_flats);
-
-  // killough 4/4/98: add colormap markers
-  W_CoalesceMarkedResource("C_START", "C_END", ns_colormaps);
+  for (int i = 0; i < arrlen(subdirs); ++i)
+  {
+    if (subdirs[i].namespace != ns_global)
+    {
+      W_CoalesceMarkedResource(subdirs[i].start_marker, subdirs[i].end_marker,
+                               subdirs[i].namespace);
+    }
+  }
 
   // [Woof!] namespace to avoid conflicts with high-resolution textures
   W_CoalesceMarkedResource("HI_START", "HI_END", ns_hires);
@@ -447,29 +476,31 @@ int W_LumpLength (int lump)
 
 void W_ReadLump(int lump, void *dest)
 {
-  lumpinfo_t *l = lumpinfo + lump;
+    lumpinfo_t *info = lumpinfo + lump;
 
 #ifdef RANGECHECK
-  if (lump >= numlumps)
-    I_Error ("W_ReadLump: %i >= numlumps",lump);
+    if (lump >= numlumps)
+    {
+        I_Error("W_ReadLump: %i >= numlumps", lump);
+    }
 #endif
 
-  if (l->data)     // killough 1/31/98: predefined lump data
-    memcpy(dest, l->data, l->size);
-  else if (l->size) // [FG] ignore empty lumps
+    if (!info->size)
     {
-      int c;
-
-      // killough 1/31/98: Reload hack (-wart) removed
-      // killough 10/98: Add flashing disk indicator
-
-      I_BeginRead(l->size);
-      lseek(l->handle, l->position, SEEK_SET);
-      c = read(l->handle, dest, l->size);
-      if (c < l->size)
-        I_Error("W_ReadLump: only read %i of %i on lump %i", c, l->size, lump);
-      I_EndRead();
+        return;
     }
+
+    if (info->data) // killough 1/31/98: predefined lump data
+    {
+        memcpy(dest, info->data, info->size);
+        return;
+    }
+
+    I_BeginRead(info->size);
+
+    info->module->Read(info->handle, dest, info->size);
+
+    I_EndRead();
 }
 
 //
@@ -494,73 +525,6 @@ void *W_CacheLumpNum(int lump, pu_tag tag)
 
 // W_CacheLumpName macroized in w_wad.h -- killough
 
-// WritePredefinedLumpWad
-// Args: Filename - string with filename to write to
-// Returns: void
-//
-// If the user puts a -dumplumps switch on the command line, we will
-// write all those predefined lumps above out into a pwad.  User
-// supplies the pwad name.
-//
-// killough 4/22/98: make endian-independent, remove tab chars
-// haleyjd 01/21/05: rewritten to use stdio
-//
-void WriteLumpWad(const char *filename, const lumpinfo_t *lumps, const size_t num_lumps)
-{
-   FILE *file;
-   char *fn;
-   
-   if(!filename || !*filename)  // check for null pointer or empty name
-      return;  // early return
-
-   fn = malloc(strlen(filename) + 5);  // we may have to add ".wad" to the name they pass
-
-   AddDefaultExtension(strcpy(fn, filename), ".wad");
-
-   // The following code writes a PWAD from the predefined lumps array
-   // How to write a PWAD will not be explained here.
-   if((file = M_fopen(fn, "wb")))
-   {
-      wadinfo_t header = { "PWAD" };
-      size_t filepos = sizeof(wadinfo_t) + num_lumps * sizeof(filelump_t);
-      int i;
-      
-      header.numlumps     = LONG(num_lumps);
-      header.infotableofs = LONG(sizeof(header));
-      
-      // write header
-      fwrite(&header, 1, sizeof(header), file);
-      
-      // write directory
-      for(i = 0; i < num_lumps; i++)
-      {
-         filelump_t fileinfo = { 0 };
-         
-         fileinfo.filepos = LONG(filepos);
-         fileinfo.size    = LONG(lumps[i].size);
-         M_CopyLumpName(fileinfo.name, lumps[i].name);
-         
-         fwrite(&fileinfo, 1, sizeof(fileinfo), file);
-
-         filepos += lumps[i].size;
-      }
-      
-      // write lumps
-      for(i = 0; i < num_lumps; i++)
-         fwrite(lumps[i].data, 1, lumps[i].size, file);
-      
-      fclose(file);
-      I_Success("Internal lumps wad, %s written, exiting\n", filename);
-   }
-   I_Error("Cannot open internal lumps wad %s for output\n", filename);
-   free(fn);
-}
-
-void WritePredefinedLumpWad(const char *filename)
-{
-    WriteLumpWad(filename, predefined_lumps, num_predefined_lumps);
-}
-
 // [FG] name of the WAD file that contains the lump
 const char *W_WadNameForLump (const int lump)
 {
@@ -608,55 +572,55 @@ int W_LumpLengthWithName(int lump, char *name)
   return W_LumpLength(lump);
 }
 
-// [FG] avoid demo lump name collisions
-void W_DemoLumpNameCollision(char **name)
+// killough 10/98: support .deh from wads
+//
+// A lump named DEHACKED is treated as plaintext of a .deh file embedded in
+// a wad (more portable than reading/writing info.c data directly in a wad).
+//
+// If there are multiple instances of "DEHACKED", we process each, in first
+// to last order (we must reverse the order since they will be stored in
+// last to first order in the chain). Passing NULL as first argument to
+// ProcessDehFile() indicates that the data comes from the lump number
+// indicated by the third argument, instead of from a file.
+
+static void ProcessInWad(int i, const char *name, void (*process)(int lumpnum),
+                         process_wad_t flag)
 {
-  const char *const safename = "DEMO1";
-  char basename[9];
-  int i, lump;
-
-  ExtractFileBase(*name, basename);
-
-  // [FG] lumps called DEMO* are considered safe
-  if (!strncasecmp(basename, safename, 4))
-  {
-    return;
-  }
-
-  lump = W_CheckNumForName(basename);
-
-  if (lump >= 0)
-  {
-    for (i = lump - 1; i >= 0; i--)
-    {
-      if (!strncasecmp(lumpinfo[i].name, basename, 8))
-      {
-        break;
-      }
-    }
-
     if (i >= 0)
     {
-      I_Printf(VB_WARNING, "Demo lump name collision detected with lump \'%.8s\' from %s.",
-              lumpinfo[i].name, W_WadNameForLump(i));
+        ProcessInWad(lumpinfo[i].next, name, process, flag);
 
-      // [FG] the DEMO1 lump is almost certainly always a demo lump
-      M_StringCopy(lumpinfo[lump].name, safename, 8);
-      *name = lumpinfo[lump].name;
+        int condition = 0;
+        if (flag & PROCESS_IWAD)
+        {
+            condition |= lumpinfo[i].wad_file == wadfiles[0];
+        }
+        if (flag & PROCESS_PWAD)
+        {
+            condition |= lumpinfo[i].wad_file != wadfiles[0];
+        }
 
-      W_InitLumpHash();
+        if (!strncasecmp(lumpinfo[i].name, name, 8)
+            && lumpinfo[i].namespace == ns_global && condition)
+        {
+            process(i);
+        }
     }
-  }
 }
 
-void W_CloseFileDescriptors(void)
+void W_ProcessInWads(const char *name, void (*process)(int lumpnum),
+                     process_wad_t flags)
 {
-  int i;
+    ProcessInWad(lumpinfo[W_LumpNameHash(name) % (unsigned)numlumps].index,
+                 name, process, flags);
+}
 
-  for (i = 0; i < array_size(handles); ++i)
-  {
-     close(handles[i]);
-  }
+void W_Close(void)
+{
+    for (int i = 0; i < arrlen(modules); ++i)
+    {
+        modules[i]->Close();
+    }
 }
 
 //----------------------------------------------------------------------------
